@@ -25,16 +25,16 @@
 
 /* global d3:false, FooTable:false */
 
-define(["jquery", "d3evolution", "footable"],
-    function ($, D3Evolution) {
+define(["jquery", "d3evolution", "d3pie", "footable"],
+    function ($, D3Evolution, D3Pie) {
         "use strict";
 
         var rrd_pie_config = {
-            header: {},
+            cornerRadius: 2,
             size: {
                 canvasWidth: 400,
                 canvasHeight: 180,
-                pieInnerRadius: "20%",
+                pieInnerRadius: "50%",
                 pieOuterRadius: "80%"
             },
             labels: {
@@ -42,148 +42,127 @@ define(["jquery", "d3evolution", "footable"],
                     format: "none"
                 },
                 inner: {
-                    hideWhenLessThanPercentage: 8
+                    hideWhenLessThanPercentage: 8,
+                    offset: 0
                 },
             },
-            misc: {
-                pieCenterOffset: {
-                    x: -120,
-                    y: 10,
-                },
-                gradient: {
-                    enabled: true,
-                },
+            padAngle: 0.02,
+            pieCenterOffset: {
+                x: -120,
+                y: 10,
+            },
+            total: {
+                enabled: true
             },
         };
 
-        var graph_options = {
-            title: "Rspamd throughput",
-            width: 1060,
-            height: 370,
-            yAxisLabel: "Message rate, msg/s",
-
-            legend: {
-                space: 140,
-                entries: [{
-                    label: "Rejected",
-                    color: "#FF0000"
-                }, {
-                    label: "Temporarily rejected",
-                    color: "#BF8040"
-                }, {
-                    label: "Subject rewritten",
-                    color: "#FF6600"
-                }, {
-                    label: "Probable spam",
-                    color: "#FFAD00"
-                }, {
-                    label: "Greylisted",
-                    color: "#436EEE"
-                }, {
-                    label: "Clean",
-                    color: "#66CC00"
-                }]
-            }
-        };
-
-        // Get selectors' current state
-        function getSelector(id) {
-            var e = document.getElementById(id);
-            return e.options[e.selectedIndex].value;
-        }
-
-        function initGraph() {
-            var graph = new D3Evolution("graph", $.extend({}, graph_options, {
-                yScale: getSelector("selYScale"),
-                type: getSelector("selType"),
-                interpolate: getSelector("selInterpolate"),
-                convert: getSelector("selConvert"),
-            }));
-            $("#selYScale").change(function () {
-                graph.yScale(this.value);
-            });
-            $("#selConvert").change(function () {
-                graph.convert(this.value);
-            });
-            $("#selType").change(function () {
-                graph.type(this.value);
-            });
-            $("#selInterpolate").change(function () {
-                graph.interpolate(this.value);
-            });
-
-            return graph;
-        }
-
-        function getRrdSummary(json, scaleFactor) {
-            var xExtents = d3.extent(d3.merge(json), function (d) { return d.x; });
-            var timeInterval = xExtents[1] - xExtents[0];
-
-            return json.map(function (curr, i) {
-                // Time intervals that don't have data are excluded from average calculation as d3.mean()ignores nulls
-                var avg = d3.mean(curr, function (d) { return d.y; });
-                // To find an integral on the whole time interval we need to convert nulls to zeroes
-                var value = d3.mean(curr, function (d) { return Number(d.y); }) * timeInterval / scaleFactor;
-                var yExtents = d3.extent(curr, function (d) { return d.y; });
-
-                return {
-                    label: graph_options.legend.entries[i].label,
-                    value: value ^ 0, // eslint-disable-line no-bitwise
-                    min: Number(yExtents[0].toFixed(6)),
-                    avg: Number(avg.toFixed(6)),
-                    max: Number(yExtents[1].toFixed(6)),
-                    last: Number(curr[curr.length - 1].y.toFixed(6)),
-                    color: graph_options.legend.entries[i].color,
-                };
-            }, []);
-        }
-
-        function initSummaryTable(tables, rows, unit) {
-            tables.rrd_summary = FooTable.init("#rrd-table", {
-                sorting: {
-                    enabled: true
-                },
-                columns: [
-                    {name:"label", title:"Action"},
-                    {name:"value", title:"Messages", defaultContent:""},
-                    {name:"min", title:"Minimum, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
-                    {name:"avg", title:"Average, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
-                    {name:"max", title:"Maximum, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
-                    {name:"last", title:"Last, " + unit},
-                ],
-                rows: rows
-            });
-        }
-
-        function drawRrdTable(tables, data, unit) {
-            var total_messages = 0;
-            var rows = data.map(function (curr, i) {
-                total_messages += curr.value;
-                return {
-                    options: {
-                        style: {
-                            color: graph_options.legend.entries[i].color
-                        }
-                    },
-                    value: curr
-                };
-            }, []);
-
-            document.getElementById("rrd-total-value").innerHTML = total_messages;
-
-            if (Object.prototype.hasOwnProperty.call(tables, "rrd_summary")) {
-                tables.rrd_summary.rows.load(rows);
-            } else {
-                initSummaryTable(tables, rows, unit);
-            }
-        }
-
         var ui = {};
         var prevUnit = "msg/s";
 
         ui.draw = function (rspamd, graphs, tables, neighbours, checked_server, type) {
+            var graph_options = {
+                title: "Rspamd throughput",
+                width: 1060,
+                height: 370,
+                yAxisLabel: "Message rate, msg/s",
+
+                legend: {
+                    space: 140,
+                    entries: rspamd.chartLegend
+                }
+            };
+
+            function initGraph() {
+                var graph = new D3Evolution("graph", $.extend({}, graph_options, {
+                    yScale: rspamd.getSelector("selYScale"),
+                    type: rspamd.getSelector("selType"),
+                    interpolate: rspamd.getSelector("selInterpolate"),
+                    convert: rspamd.getSelector("selConvert"),
+                }));
+                $("#selYScale").change(function () {
+                    graph.yScale(this.value);
+                });
+                $("#selConvert").change(function () {
+                    graph.convert(this.value);
+                });
+                $("#selType").change(function () {
+                    graph.type(this.value);
+                });
+                $("#selInterpolate").change(function () {
+                    graph.interpolate(this.value);
+                });
+
+                return graph;
+            }
+
+            function getRrdSummary(json, scaleFactor) {
+                var xExtents = d3.extent(d3.merge(json), function (d) { return d.x; });
+                var timeInterval = xExtents[1] - xExtents[0];
+
+                var total = 0;
+                var rows = json.map(function (curr, i) {
+                    // Time intervals that don't have data are excluded from average calculation as d3.mean()ignores nulls
+                    var avg = d3.mean(curr, function (d) { return d.y; });
+                    // To find an integral on the whole time interval we need to convert nulls to zeroes
+                    var value = d3.mean(curr, function (d) { return Number(d.y); }) * timeInterval / scaleFactor ^ 0; // eslint-disable-line no-bitwise
+                    var yExtents = d3.extent(curr, function (d) { return d.y; });
+
+                    total += value;
+                    return {
+                        label: graph_options.legend.entries[i].label,
+                        value: value,
+                        min: Number(yExtents[0].toFixed(6)),
+                        avg: Number(avg.toFixed(6)),
+                        max: Number(yExtents[1].toFixed(6)),
+                        last: Number(curr[curr.length - 1].y.toFixed(6)),
+                        color: graph_options.legend.entries[i].color,
+                    };
+                }, []);
+
+                return {
+                    rows: rows,
+                    total: total
+                };
+            }
+
+            function initSummaryTable(rows, unit) {
+                tables.rrd_summary = FooTable.init("#rrd-table", {
+                    sorting: {
+                        enabled: true
+                    },
+                    columns: [
+                        {name:"label", title:"Action"},
+                        {name:"value", title:"Messages", defaultContent:""},
+                        {name:"min", title:"Minimum, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
+                        {name:"avg", title:"Average, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
+                        {name:"max", title:"Maximum, <span class=\"unit\">" + unit + "</span>", defaultContent:""},
+                        {name:"last", title:"Last, " + unit},
+                    ],
+                    rows: rows.map(function (curr, i) {
+                        return {
+                            options: {
+                                style: {
+                                    color: graph_options.legend.entries[i].color
+                                }
+                            },
+                            value: curr
+                        };
+                    }, [])
+                });
+            }
+
+            function drawRrdTable(rows, unit) {
+                if (Object.prototype.hasOwnProperty.call(tables, "rrd_summary")) {
+                    $.each(tables.rrd_summary.rows.all, function (i, row) {
+                        row.val(rows[i], false, true);
+                    });
+                } else {
+                    initSummaryTable(rows, unit);
+                }
+            }
+
             function updateWidgets(data) {
-                var rrd_summary = [];
+                var rrd_summary = {rows:[]};
                 var unit = "msg/s";
 
                 if (data) {
@@ -201,21 +180,19 @@ define(["jquery", "d3evolution", "footable"],
                     }
 
                     rrd_summary = getRrdSummary(data, scaleFactor);
-                    graphs.rrd_pie = rspamd.drawPie(graphs.rrd_pie,
-                        "rrd-pie",
-                        rrd_summary,
-                        rrd_pie_config);
-                } else if (graphs.rrd_pie) {
-                    graphs.rrd_pie.destroy();
                 }
 
+                if (!graphs.rrd_pie) graphs.rrd_pie = new D3Pie("rrd-pie", rrd_pie_config);
+                graphs.rrd_pie.data(rrd_summary.rows);
+
                 graphs.graph.data(data);
                 if (unit !== prevUnit) {
                     graphs.graph.yAxisLabel("Message rate, " + unit);
                     $(".unit").text(unit);
                     prevUnit = unit;
                 }
-                drawRrdTable(tables, rrd_summary, unit);
+                drawRrdTable(rrd_summary.rows, unit);
+                document.getElementById("rrd-total-value").innerHTML = rrd_summary.total;
             }
 
             if (!graphs.graph) {
@@ -259,24 +236,25 @@ define(["jquery", "d3evolution", "footable"],
                     }
                     updateWidgets(data);
                 },
+                complete: function () { $("#refresh").removeAttr("disabled").removeClass("disabled"); },
                 errorMessage: "Cannot receive throughput data",
                 errorOnceId: "alerted_graph_",
                 data: {type:type}
             });
         };
 
-        ui.setup = function () {
+        ui.setup = function (rspamd) {
             // Handling mouse events on overlapping elements
             $("#rrd-pie").mouseover(function () {
-                $("#rrd-pie").css("z-index", "200");
+                $("#rrd-pie,#rrd-pie-tooltip").css("z-index", "200");
                 $("#rrd-table_toggle").css("z-index", "300");
             });
             $("#rrd-table_toggle").mouseover(function () {
-                $("#rrd-pie").css("z-index", "0");
+                $("#rrd-pie,#rrd-pie-tooltip").css("z-index", "0");
                 $("#rrd-table_toggle").css("z-index", "0");
             });
 
-            return getSelector("selData");
+            return rspamd.getSelector("selData");
         };
 
         return ui;
